#include"MeshLoader.h"


std::vector<Node>& MeshLoader::getNode()
{
	return node;
}

std::vector< FiniteElement>& MeshLoader::getFiniteElement()
{
        return finelem;
}

std::vector< BoundaryFiniteElement>& MeshLoader::getBoundaryFiniteElement()
{
        return boundelem;
}


std::vector< FiniteElement> MeshLoader::getFiniteElementByIdNodes(sizeT id1, sizeT id2, sizeT id3)
{
	std::vector<  FiniteElement> finiteElements;
        std::vector<  FiniteElement>::iterator current_iter = finelem.begin();

        while (current_iter != finelem.end())
	{
                current_iter = std::find_if(current_iter, finelem.end(), [id1, id2, id3](const FiniteElement& elem)
		{
                        return (std::find(elem.id_node.begin(), elem.id_node.end(), id1) != elem.id_node.end()
                                && std::find(elem.id_node.begin(), elem.id_node.end(), id2) != elem.id_node.end()
                                && std::find(elem.id_node.begin(), elem.id_node.end(), id3) != elem.id_node.end());
		});
                if (current_iter != finelem.end())
		{
			finiteElements.push_back(*current_iter);
			++current_iter;
		}
	}

	return finiteElements;
}


std::vector<FiniteElement> MeshLoader::getFiniteElementById(sizeT IdMaterial)
{
        std::vector< FiniteElement> finelem_inMaterial;
        std::for_each(finelem.begin(), finelem.end(), [&](const FiniteElement& elem)
        {
                if (elem.id_material == IdMaterial)
                {
                        finelem_inMaterial.push_back(elem);
                }
        });
        return finelem_inMaterial;
}


std::vector<Node> MeshLoader::getNodeByIdSurface(sizeT IdSurface)
{
        std::vector<Node> nodeInSurface;
        std::for_each(boundelem.begin(), boundelem.end(), [&](const BoundaryFiniteElement& elem)
        {
                if (elem.id_surface == IdSurface)
                {
                        for (auto iterId = elem.id_node.begin(); iterId != elem.id_node.end(); iterId++)
                        {
                                if( std::find(nodeInSurface.begin(), nodeInSurface.end() , node[*iterId - 1]) == nodeInSurface.end() )
                                nodeInSurface.push_back(node[*iterId - 1]);
                        }
                }
        });

        return nodeInSurface;
}


std::vector < FiniteElement> MeshLoader::getFiniteElementByEdge(sizeT id1, sizeT id2)
{
	std::vector< FiniteElement> finiteElements;


        std::vector<FiniteElement>::iterator current_iter = finelem.begin();
        while (current_iter != finelem.end())
	{
                current_iter = std::find_if(current_iter, finelem.end(), [id1, id2](const FiniteElement& elem)
		{
                        return (std::find(elem.id_node.begin(), elem.id_node.end(), id1) != elem.id_node.end() &&
                                std::find(elem.id_node.begin(), elem.id_node.end(), id2) != elem.id_node.end());
		});
                if (current_iter != finelem.end())
		{
			finiteElements.push_back(*current_iter);
			++current_iter;
		}
	}

	return finiteElements;
}

std::vector <BoundaryFiniteElement>  MeshLoader::getBoundaryFiniteElementByIdSuface(sizeT IdSurface)
{
        std::vector< BoundaryFiniteElement> bounelem_inSurface;
        std::for_each(boundelem.begin(), boundelem.end(), [&](const BoundaryFiniteElement& elem)
	{
                if (elem.id_surface == IdSurface)
		{
                        bounelem_inSurface.push_back(elem);
		}
	});

        return bounelem_inSurface;
}


void MeshLoader::print() const
{
	std::cout << "Nodes:"<<std::endl;

        std::for_each(node.begin(), node.end(), [](const Node& elem)
	{
                std::cout << (elem);
	});
	std::cout << std::endl;

	std::cout << "Finite Elements:" << std::endl;
        std::for_each(finelem.begin(), finelem.end(), [](const FiniteElement& elem)
	{
                std::cout << elem;
	});
	std::cout << std::endl;

	std::cout << "Boundary finite elements:" << std::endl;
        std::for_each(boundelem.begin(), boundelem.end(), [](const BoundaryFiniteElement& elem)
	{
                std::cout << elem;
	});
}


std::vector<std::set<sizeT>>  MeshLoader::getNeighboursNodes()
{
        std::vector < std::set<sizeT>> result(node.size());

        for (auto finiteelem : finelem)
	{
                for (auto id_node : finiteelem.id_node)
		{
                        result[id_node - 1].insert(finiteelem.id_node.begin(), finiteelem.id_node.end());
			result[id_node - 1].erase(id_node);
		}
	}

	return result;
}


void MeshLoader::InsertMidNodes()
{
	std::unordered_set<Edge, HashEdge> container_edge;

        for (FiniteElement& finiteElem : finelem)
	{
                std::vector<sizeT> currentId_node = finiteElem.id_node;

                for (auto i = currentId_node.begin(); i != currentId_node.end() - 1 ; i++)
		{
                        for (auto j = i + 1; j != currentId_node.end(); j++)
			{
				Edge current_edge(*i, *j);
				std::unordered_set<Edge, HashEdge>::iterator search_edge = container_edge.find(current_edge);
				Node middle((node[*i].coord[0] + node[*j].coord[0]) / 2, (node[*i].coord[1] + node[*j].coord[1]) / 2, (node[*i].coord[2] + node[*j].coord[2]) / 2, node.size() + 1, false);

				if (search_edge == container_edge.end())
				{
					current_edge.middle_node = node.size() + 1;
					container_edge.insert(current_edge);
					node.push_back(middle);
					finiteElem.id_node.push_back(middle.id);
				}
				else
				{
					finiteElem.id_node.push_back(current_edge.second_node);
				}

			}
		}
	}

        for (BoundaryFiniteElement& boundary : boundelem)
	{
                std::vector<sizeT> currentId_node = boundary.id_node;
                for (std::vector<sizeT>::iterator i = currentId_node.begin(); i != currentId_node.end() - 1; i++)
		{
                        for (std::vector<sizeT>::iterator j = i + 1; j != currentId_node.end(); j++)
			{
				std::unordered_set<Edge, HashEdge>::iterator current_edge = container_edge.find(Edge(*i, *j));

				if (current_edge != container_edge.end())
				{
					boundary.id_node.push_back(current_edge->middle_node);
				}
			}
		}
	}
}
