#include<iomanip>
#include"Structure.h"

std::ostream& operator<<(std::ostream& ostr, const Node& node)
{
        ostr << std::setw(8) << node.id
		<< std::setw(12) << std::setprecision(6) << std::fixed << node.coord[0]
		<< std::setw(12) << std::setprecision(6) << std::fixed << node.coord[1]
		<< std::setw(12) << std::setprecision(6) << std::fixed << node.coord[2]
		<< std::setw(12) <<std::boolalpha << node.top << std::endl;

        return ostr;
}

std::ostream& operator<<(std::ostream& ostr, const FiniteElement& fin)
{
        ostr << std::setw(8) << fin.id_material << std::setw(8) << fin.id_node[0] << std::setw(8) << fin.id_node[1] << std::setw(8) << fin.id_node[2] << std::setw(8) << fin.id_node[3]<<std::endl;
        return ostr;
}

std::ostream& operator<<(std::ostream& ostr, const BoundaryFiniteElement& bound)
{

        ostr << std::setw(8) << bound.id_surface << std::setw(8) << bound.id_node[0] << std::setw(8) << bound.id_node[1] << std::setw(8) << bound.id_node[2] << std::endl ;
        return ostr;
}

bool Node::operator==(const Node& _node) const
{
	return id == _node.id;
}

bool Node::operator!=(const Node& _node) const
{
	return id != _node.id;
}

bool FiniteElement::operator==(const FiniteElement& _finelem) const
{
        return id == _finelem.id;
}

bool FiniteElement::operator!=(const FiniteElement& _finelem) const
{
        return id != _finelem.id;
}

bool BoundaryFiniteElement::operator==(const BoundaryFiniteElement& _Boundelem) const
{
        return id == _Boundelem.id;
}

bool BoundaryFiniteElement::operator!=(const BoundaryFiniteElement& _Boundelem) const
{
        return id != _Boundelem.id;
}

Node::Node(double X, double Y, double Z, sizeT Id, bool Top )
{
        coord[0] = X;
        coord[1] = Y;
        coord[2] = Z;
        id = Id;
        top = Top;

}

FiniteElement::FiniteElement(sizeT IdMaterial, sizeT N1, sizeT N2, sizeT N3, sizeT N4, sizeT Id)  : id_node (4)
{

        id_material = IdMaterial;
	id_node[0] = N1;
	id_node[1] = N2;
	id_node[2] = N3;
	id_node[3] = N4;
        id = Id;
}

BoundaryFiniteElement::BoundaryFiniteElement(sizeT IdSurface, sizeT N1, sizeT N2, sizeT N3, sizeT Id) : id_node(3)
{
        id_surface = IdSurface;
	id_node[0] = N1;
	id_node[1] = N2;
	id_node[2] = N3;
        id = Id;
}

Edge::Edge(sizeT node1, sizeT node2)
{
	first_node = node1;

	second_node = node2;
}

bool Edge::operator==(const Edge& _edge) const
{
        return (first_node == _edge.first_node && second_node == _edge.second_node) || (first_node == _edge.second_node && second_node == _edge.first_node);
}

sizeT HashEdge::operator()(const Edge& edge) const
{

        sizeT seed = 0;

	if (edge.first_node > edge.second_node)
	{
                seed ^= std::hash<int>() (edge.first_node) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
                seed ^= std::hash<int>() (edge.second_node) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
	}
	else
	{
                seed ^= std::hash<int>() (edge.second_node) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
                seed ^= std::hash<int>() (edge.first_node) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
	}


        return seed;
}
